@@ -1,11 +1,11 @@
 ##' @method ggplot_add fruit_plot
 ##' @importFrom utils modifyList
-##' @importFrom ggplot2 aes aes_ aes_string scale_color_manual
+##' @importFrom ggplot2 aes scale_color_manual
 ##' @importFrom rlang as_name quo_name
 ##' @importFrom ggnewscale new_scale_color
 ##' @author Shuangbin Xu
 ##' @export
-ggplot_add.fruit_plot <- function(object, plot, object_name){
+ggplot_add.fruit_plot <- function(object, plot, object_name, ...){
     object <- check_plotdata(object=object, plot=plot)
     object <- check_subset_aes(object=object)
     object <- build_new_data(object=object, plot=plot)
@@ -15,12 +15,12 @@ ggplot_add.fruit_plot <- function(object, plot, object_name){
     xid <- res[[2]]
     layout <- get_layout(plot)
     flagreverse <- check_reverse(plot=plot)
-    if (layout=="inward_circular" || flagreverse){
+    if (layout %in% c("inward_circular", "dendrogram") || flagreverse){
         orientation <- -1
     }else{
         orientation <- 1
     }
-    offset <- get_offset(plot$data$x, object$offset)
+    offset <- get_offset(plot$data, object$offset)
     if ("xmaxtmp" %in% colnames(plot$data)){
         hexpand2 <- max(abs(plot$data$xmaxtmp), na.rm=TRUE) + offset
     }else{
@@ -61,7 +61,7 @@ ggplot_add.fruit_plot <- function(object, plot, object_name){
             newxexpand <- max(abs(dat[[paste0("new_", xid)]]), na.rm=TRUE)
         }else{
             if (!"hexpand" %in% names(object$params$position)){
-                dat[[paste0("new_", xid)]] <- data.frame(plot$data, check.names=FALSE)[match(dat$label,plot$data$label),"x"]
+                dat[[paste0("new_", xid)]] <- data.frame(plot$data, check.names=FALSE)[match(dat$y,plot$data$y),"x"]
             }else{
                 dat[[paste0("new_", xid)]] <- 0
             }
@@ -69,6 +69,9 @@ ggplot_add.fruit_plot <- function(object, plot, object_name){
             object <- .set_pwidth2width(object)
         }
     }
+    if (layout == 'dendrogram'){
+        offset <- abs(offset)
+    }
     if ("xmaxtmp" %in% colnames(plot$data)){
         plot$data$xmaxtmp <- plot$data$xmaxtmp + newxexpand + offset
     }else{
@@ -76,7 +79,7 @@ ggplot_add.fruit_plot <- function(object, plot, object_name){
     }
     if ("hexpand" %in% names(object$params$position)){
         if (is.na(object$params$position$hexpand)){
-            if (orientation < 0){
+            if (orientation < 0 && layout!='dendrogram'){
                 hexpand2 <- abs(hexpand2)
             }
             object$params$position$hexpand <- hexpand2
@@ -85,17 +88,17 @@ ggplot_add.fruit_plot <- function(object, plot, object_name){
     tmpangle <- dat$angle
     if (object$geomname=="geom_star"){
         dat$angle <- adjust_angle(layout=layout, angle=tmpangle)
-        object$mapping = modifyList(object$mapping, aes_(angle=~angle))
+        object$mapping = modifyList(object$mapping, aes(angle=!!sym("angle")))
     }
     if (object$geomname=="geom_text"){
         dat$angle <- adjust_text_angle(layout=layout, angle=tmpangle)
-        object$mapping = modifyList(object$mapping, aes_(angle=~angle))
+        object$mapping = modifyList(object$mapping, aes(angle=!!sym("angle")))
     }
     if (object$geomname %in% c(dodpos, densitypos)){
-        object$mapping = modifyList(object$mapping, aes(color=factor(eval(parse(text="y")))))
+        object$mapping = modifyList(object$mapping, aes(color=factor(!!sym("y"))))
         plot <- plot + new_scale_color()
     }
-    object$mapping = modifyList(object$mapping, aes_string(x=paste0("new_",xid)))
+    object$mapping = modifyList(object$mapping, aes(x=!!sym(paste0("new_",xid))))
     params <- c(list(data=dat, mapping=object$mapping, inherit.aes=object$inherit.aes), object$params)
     obj <- do.call(object$geom, params)
     if (object$axis.params$axis != "none"){
@@ -125,14 +128,14 @@ ggplot_add.fruit_plot <- function(object, plot, object_name){
         .generate_colour_warning(plot)
         obj <- list(obj, scale_color_manual(values=c(rep("black", length(dat$y))), guide="none"), new_scale_color())
     }
-    ggplot_add(obj, plot, object_name)
+    ggplot_add(obj, plot, object_name, ...)
 }
 
 ##' @method ggplot_add layer_fruits
 ##' @author Shuangbin Xu
 ##' @export
-ggplot_add.layer_fruits <- function(object, plot, object_name){
-    offset <- get_offset(plot$data$x, object[[1]]$offset)
+ggplot_add.layer_fruits <- function(object, plot, object_name, ...){
+    offset <- get_offset(plot$data, object[[1]]$offset)
     if ("xmaxtmp" %in% colnames(plot$data)){
         hexpand2 <- max(abs(plot$data$xmaxtmp), na.rm=TRUE) + offset
     }else{
@@ -142,7 +145,7 @@ ggplot_add.layer_fruits <- function(object, plot, object_name){
     for (o in object){
         n = n + 1
         if (inherits(o, "fruit_plot")){
-            offset.i <- get_offset(plot$data$x, o$offset)
+            offset.i <- get_offset(plot$data, o$offset)
             if (offset != offset.i && n != 1){
                 cli::cli_alert_warning(c("The {.arg offset} argument of {.fun geom_fruit} layers in {.fun geom_fruit_list} is different.", 
                               "Please keep it consistent if you want to display these layers on the same position."))
@@ -176,7 +179,7 @@ create_text_data <- function(data, origin, newxid, flagrev){
 #' @importFrom cli cli_abort
 check_plotdata <- function(object, plot){
     if (is.null(object$data)){
-        object$mapping <- modifyList(object$mapping, aes_(y=~y))
+        object$mapping <- modifyList(object$mapping, aes(y=!!sym("y")))
         object$data <- plot$data[plot$data$isTip,]
                                  #!colnames(plot$data) %in% c("parent", "node", "branch.length", "isTip", "x", "branch"), 
                                  #drop=FALSE]
@@ -187,7 +190,7 @@ check_plotdata <- function(object, plot){
             cli_abort("Data function must return a data.frame")
         }
         object$data <- object$data[object$data$isTip,]
-        object$mapping <- modifyList(object$mapping, aes_(y=~y))
+        object$mapping <- modifyList(object$mapping, aes(y=!!sym("y")))
         object$datanull <- TRUE
     }else{
         object$datanull <- FALSE
@@ -195,8 +198,16 @@ check_plotdata <- function(object, plot){
     return (object)
 }
 
-get_offset <- function(vnum, ratio){
-    offset <- ratio*(max(vnum, na.rm=TRUE) - min(vnum, na.rm=TRUE))
+get_offset <- function(da, ratio){
+    vnum <- da$x
+    tmp <- max(vnum, na.rm=TRUE) - min(vnum, na.rm=TRUE)
+    flag <- da[da$isTip, 'x',drop=TRUE] <= 0
+    if (all(flag)){
+        offset <- -tmp - ratio * tmp
+    }else{
+        offset <- ratio * tmp
+    }
+    return(offset)
 }
 
 build_new_data <- function(object, plot){
@@ -209,8 +220,9 @@ build_new_data <- function(object, plot){
                          " are/is the same to tree data, the tree data column names are : ",
                          paste0(colnames(origindata), collapse=", "), "."))
         }
-        object$data <- merge(origindata, object$data, by.x="label", by.y=as_name(object$mapping$y))
-        object$mapping <- modifyList(object$mapping, aes_(y=~y))
+        #object$data <- merge(origindata, object$data, by.x="label", by.y=as_name(object$mapping$y))
+        object$data <- merge(object$data, origindata, by.x=as_name(object$mapping$y), by.y="label")
+        object$mapping <- modifyList(object$mapping, aes(y=!!sym("y")))
     }
     object$datanull <- NULL
     return(object)
@@ -274,6 +286,11 @@ stackpos <- c("geom_bar", "geom_barh", "geom_bar_pattern", "geom_col", "geom_col
 
 densitypos <- c("geom_density_ridges", "geom_density_ridges2", "geom_density_ridges_gradient")
 
+dodpos <- c(dodpos, paste0(dodpos, "_interactive"))
+idepos <- c(idepos, paste0(idepos, "_interactive"))
+stackpos <- c(stackpos, paste0(stackpos, "_interactive"))
+densitypos <- c(densitypos, paste0(densitypos, "_interactive"))
+
 check_reverse <- function(plot){
     flag <- unlist(lapply(plot$scales$scales, 
                           function(x){
@@ -281,6 +298,8 @@ check_reverse <- function(plot){
                           }))
     if (!all(flag)){return(FALSE)}
     flag <- plot$scales$scales[[which(flag)]]$trans$name=="reverse" && inherits(plot$coordinates, "CoordPolar")
+    #flag2 <- inherits(plot$coordinates, "CoordPolar") || inherits(plot$coordinates, "CoordFlip")
+    #flag <- flag1 && flag2
     if (is.na(flag)){return(FALSE)}
     return(flag)
 }
